require "import"
import "android.widget.*"
import "com.androlua.*"
import "android.view.View"
import "android.media.MediaPlayer"
import "android.net.Uri"
import "android.content.Context"
import "android.app.DownloadManager"
import "android.os.Environment"
import "android.text.InputType"
import "java.io.File"
import "android.content.Intent"

-- Load the languages module
local languages = require("lang")

-- Function to generate text-to-speech using Google API
function generateTextToAudio(text, lang, callback)
  local apiUrl = "https://googletexttospeech-apihubforblind.onrender.com/?text=" .. text .. "&lang=" .. lang
  Http.get(apiUrl, nil, function(code, content)
    if code == 200 then
      callback(apiUrl, nil)
    else
      callback(nil, "Error generating audio. Status code: " .. code)
    end
  end)
end

-- Function to download audio
function downloadAudio(audioUrl)
  local baseFolderPath = Environment.getExternalStorageDirectory().getPath() .. "/Download/Jieshuo Materials Hub/Google text-to-audio generator."
  local folder = File(baseFolderPath)
  if not folder.exists() then
    folder.mkdirs()
  end
  local fileName = "Audio_" .. os.time() .. ".mp3"
  local file = File(baseFolderPath, fileName)
  local request = DownloadManager.Request(Uri.parse(audioUrl))
  request.setTitle("Generated Audio")
  request.setDescription("Downloading generated audio")
  request.setNotificationVisibility(DownloadManager.Request.VISIBILITY_VISIBLE_NOTIFY_COMPLETED)
  request.setDestinationUri(Uri.fromFile(file))
  local downloadManager = this.getSystemService(Context.DOWNLOAD_SERVICE)
  local downloadId = downloadManager.enqueue(request)
  service.speak("Audio is being saved at: " .. file.getAbsolutePath())
  return downloadId
end

-- Create UI layout
layout = {
  ScrollView,
  layout_width = "fill",
  layout_height = "fill",
  {
    LinearLayout,
    orientation = "vertical",
    layout_width = "fill",
    padding = "16dp",
    layout_width = "fill",
    layout_height = "wrap_content",
    { Spinner, id = "languageSpinner", layout_width = "fill", layout_height = "wrap_content" },
    { EditText, id = "textInput", hint = "Enter your text", layout_width = "fill", layout_height = "wrap_content", padding = "10dp", gravity = "top", lines = 10, inputType = InputType.TYPE_CLASS_TEXT | InputType.TYPE_TEXT_FLAG_MULTI_LINE },
    { Button, id = "generateAudioButton", text = "Generate Audio", layout_width = "fill", layout_height = "wrap_content" },
    { TextView, id = "resultText", text = "", layout_width = "fill", layout_height = "wrap_content", padding = "10dp" },
    { Button, id = "playPauseAudioButton", text = "Play Audio", layout_width = "wrap_content", layout_height = "wrap_content", visibility = View.GONE },
    { Button, id = "downloadButton", text = "Download Audio", layout_width = "wrap_content", layout_height = "wrap_content", visibility = View.GONE },
    { Button, id = "telegramButton", text = "Join Telegram Channel", layout_width = "fill", layout_height = "wrap_content" },
    { Button, id = "exitButton", text = "Exit", layout_width = "fill", layout_height = "wrap_content" }
  }
}

dlg = LuaDialog(this)
dlg.setTitle("Google text-to-audio generator.")
dlg.setView(loadlayout(layout))

-- Load languages into Spinner
local languageNames = {}
local languageMap = {}

for _, lang in ipairs(languages) do
  table.insert(languageNames, lang.name)
  languageMap[lang.name] = lang.code
end

local languageAdapter = ArrayAdapter(this, android.R.layout.simple_spinner_dropdown_item, String(languageNames))
languageSpinner.setAdapter(languageAdapter)
languageSpinner.setSelection(0)

local mediaPlayer = MediaPlayer()

function updateResultText(result)
  resultText.text = result
end

function toggleAudioControls(show)
  playPauseAudioButton.visibility = show and View.VISIBLE or View.GONE
  downloadButton.visibility = show and View.VISIBLE or View.GONE
end

generateAudioButton.onClick = function()
  local text = textInput.text
  local selectedLanguage = languageMap[languageSpinner.getSelectedItem()]
  if text and #text > 0 then
    service.speak("Generating audio, please wait.")
    updateResultText("Generating audio...")
    toggleAudioControls(false)
    generateTextToAudio(text, selectedLanguage, function(audioUrl, error)
      if error then
        updateResultText("Error: " .. error)
        toggleAudioControls(false)
      else
        service.speak("Audio generated.")
        updateResultText("Audio generated successfully!")
        toggleAudioControls(true)
        playPauseAudioButton.tag = audioUrl
        downloadButton.tag = audioUrl
      end
    end)
  else
    service.speak("Please enter your text.")
    updateResultText("Please enter text for audio generation")
    toggleAudioControls(false)
  end
end

playPauseAudioButton.onClick = function()
  local audioUrl = playPauseAudioButton.tag
  if audioUrl then
    if mediaPlayer.isPlaying() then
      mediaPlayer.pause()
      playPauseAudioButton.text = "Play Audio"
      updateResultText("Audio paused.")
    else
      mediaPlayer.reset()
      mediaPlayer.setDataSource(audioUrl)
      mediaPlayer.prepareAsync()
      mediaPlayer.setOnPreparedListener(MediaPlayer.OnPreparedListener {onPrepared = function()
        mediaPlayer.start()
        playPauseAudioButton.text = "Pause Audio"
        updateResultText("Playing audio...")
      end})
      mediaPlayer.setOnCompletionListener(MediaPlayer.OnCompletionListener {onCompletion = function()
        playPauseAudioButton.text = "Play Audio"
        updateResultText("Audio playback finished.")
      end})
    end
  else
    service.speak("No audio URL available.")
  end
end

downloadButton.onClick = function()
  local audioUrl = downloadButton.tag
  if audioUrl then
    downloadAudio(audioUrl)
  else
    service.speak("No audio URL available.")
  end
end

telegramButton.onClick = function()
  local url = "https://t.me/jieshuoofficial/"
  local intent = Intent(Intent.ACTION_VIEW)
  intent.setData(Uri.parse(url))
  this.startActivity(intent)
  dlg.dismiss()
end

exitButton.onClick = function()
  if mediaPlayer.isPlaying() then
    mediaPlayer.stop()
  end
  mediaPlayer.release()
  dlg.dismiss()
end

dlg.show()